"""Shell completion script generation operations."""

from erk_shared.gateway.completion.abc import Completion as Completion
from erk_shared.gateway.completion.fake import FakeCompletion as FakeCompletion
from erk_shared.gateway.completion.real import RealCompletion as RealCompletion
