"""Abstract base class for GitHub Actions admin operations.

Re-exports from erk_shared for backwards compatibility.
"""

from erk_shared.github_admin.abc import AuthStatus as AuthStatus
from erk_shared.github_admin.abc import GitHubAdmin as GitHubAdmin
